from __future__ import unicode_literals


from datetime import datetime
from dateutil.tz import tzutc
import boto3
from freezegun import freeze_time
import requests
import sure  # noqa
from botocore.exceptions import ClientError

from moto.packages.responses import responses
from moto import mock_apigateway, settings


@freeze_time("2015-01-01")
@mock_apigateway
def test_create_and_get_rest_api():
    client = boto3.client('apigateway', region_name='us-west-2')

    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']

    response = client.get_rest_api(
        restApiId=api_id
    )

    response.pop('ResponseMetadata')
    response.pop('createdDate')
    response.should.equal({
        'id': api_id,
        'name': 'my_api',
        'description': 'this is my api',
    })


@mock_apigateway
def test_list_and_delete_apis():
    client = boto3.client('apigateway', region_name='us-west-2')

    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']
    client.create_rest_api(
        name='my_api2',
        description='this is my api2',
    )

    response = client.get_rest_apis()
    len(response['items']).should.equal(2)

    client.delete_rest_api(
        restApiId=api_id
    )

    response = client.get_rest_apis()
    len(response['items']).should.equal(1)


@mock_apigateway
def test_create_resource():
    client = boto3.client('apigateway', region_name='us-west-2')
    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']

    resources = client.get_resources(restApiId=api_id)
    root_id = [resource for resource in resources[
        'items'] if resource['path'] == '/'][0]['id']

    root_resource = client.get_resource(
        restApiId=api_id,
        resourceId=root_id,
    )
    # this is hard to match against, so remove it
    root_resource['ResponseMetadata'].pop('HTTPHeaders', None)
    root_resource['ResponseMetadata'].pop('RetryAttempts', None)
    root_resource.should.equal({
        'path': '/',
        'id': root_id,
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'resourceMethods': {
            'GET': {}
        }
    })

    response = client.create_resource(
        restApiId=api_id,
        parentId=root_id,
        pathPart='/users',
    )

    resources = client.get_resources(restApiId=api_id)['items']
    len(resources).should.equal(2)
    non_root_resource = [
        resource for resource in resources if resource['path'] != '/'][0]

    response = client.delete_resource(
        restApiId=api_id,
        resourceId=non_root_resource['id']
    )

    len(client.get_resources(restApiId=api_id)['items']).should.equal(1)


@mock_apigateway
def test_child_resource():
    client = boto3.client('apigateway', region_name='us-west-2')
    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']

    resources = client.get_resources(restApiId=api_id)
    root_id = [resource for resource in resources[
        'items'] if resource['path'] == '/'][0]['id']

    response = client.create_resource(
        restApiId=api_id,
        parentId=root_id,
        pathPart='users',
    )
    users_id = response['id']

    response = client.create_resource(
        restApiId=api_id,
        parentId=users_id,
        pathPart='tags',
    )
    tags_id = response['id']

    child_resource = client.get_resource(
        restApiId=api_id,
        resourceId=tags_id,
    )
    # this is hard to match against, so remove it
    child_resource['ResponseMetadata'].pop('HTTPHeaders', None)
    child_resource['ResponseMetadata'].pop('RetryAttempts', None)
    child_resource.should.equal({
        'path': '/users/tags',
        'pathPart': 'tags',
        'parentId': users_id,
        'id': tags_id,
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'resourceMethods': {'GET': {}},
    })


@mock_apigateway
def test_create_method():
    client = boto3.client('apigateway', region_name='us-west-2')
    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']

    resources = client.get_resources(restApiId=api_id)
    root_id = [resource for resource in resources[
        'items'] if resource['path'] == '/'][0]['id']

    client.put_method(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        authorizationType='none',
    )

    response = client.get_method(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET'
    )

    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response.should.equal({
        'httpMethod': 'GET',
        'authorizationType': 'none',
        'ResponseMetadata': {'HTTPStatusCode': 200}
    })


@mock_apigateway
def test_create_method_response():
    client = boto3.client('apigateway', region_name='us-west-2')
    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']

    resources = client.get_resources(restApiId=api_id)
    root_id = [resource for resource in resources[
        'items'] if resource['path'] == '/'][0]['id']

    client.put_method(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        authorizationType='none',
    )

    response = client.get_method(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET'
    )

    response = client.put_method_response(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        statusCode='200',
    )
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response.should.equal({
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'statusCode': '200'
    })

    response = client.get_method_response(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        statusCode='200',
    )
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response.should.equal({
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'statusCode': '200'
    })

    response = client.delete_method_response(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        statusCode='200',
    )
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response.should.equal({'ResponseMetadata': {'HTTPStatusCode': 200}})


@mock_apigateway
def test_integrations():
    client = boto3.client('apigateway', region_name='us-west-2')
    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']

    resources = client.get_resources(restApiId=api_id)
    root_id = [resource for resource in resources[
        'items'] if resource['path'] == '/'][0]['id']

    client.put_method(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        authorizationType='none',
    )

    client.put_method_response(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        statusCode='200',
    )

    response = client.put_integration(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        type='HTTP',
        uri='http://httpbin.org/robots.txt',
    )
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response.should.equal({
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'httpMethod': 'GET',
        'integrationResponses': {
            '200': {
                'responseTemplates': {
                    'application/json': None
                },
                'statusCode': 200
            }
        },
        'type': 'HTTP',
        'uri': 'http://httpbin.org/robots.txt'
    })

    response = client.get_integration(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET'
    )
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response.should.equal({
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'httpMethod': 'GET',
        'integrationResponses': {
            '200': {
                'responseTemplates': {
                    'application/json': None
                },
                'statusCode': 200
            }
        },
        'type': 'HTTP',
        'uri': 'http://httpbin.org/robots.txt'
    })

    response = client.get_resource(
        restApiId=api_id,
        resourceId=root_id,
    )
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response['resourceMethods']['GET']['methodIntegration'].should.equal({
        'httpMethod': 'GET',
        'integrationResponses': {
            '200': {
                'responseTemplates': {
                    'application/json': None
                },
                'statusCode': 200
            }
        },
        'type': 'HTTP',
        'uri': 'http://httpbin.org/robots.txt'
    })

    client.delete_integration(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET'
    )

    response = client.get_resource(
        restApiId=api_id,
        resourceId=root_id,
    )
    response['resourceMethods']['GET'].shouldnt.contain("methodIntegration")

    # Create a new integration with a requestTemplates config

    client.put_method(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='POST',
        authorizationType='none',
    )

    templates = {
        # example based on
        # http://docs.aws.amazon.com/apigateway/latest/developerguide/api-as-kinesis-proxy-export-swagger-with-extensions.html
        'application/json': "{\n    \"StreamName\": \"$input.params('stream-name')\",\n    \"Records\": []\n}"
    }
    test_uri = 'http://example.com/foobar.txt'
    response = client.put_integration(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='POST',
        type='HTTP',
        uri=test_uri,
        requestTemplates=templates
    )
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response['ResponseMetadata'].should.equal({'HTTPStatusCode': 200})

    response = client.get_integration(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='POST'
    )
    response['uri'].should.equal(test_uri)
    response['requestTemplates'].should.equal(templates)


@mock_apigateway
def test_integration_response():
    client = boto3.client('apigateway', region_name='us-west-2')
    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']

    resources = client.get_resources(restApiId=api_id)
    root_id = [resource for resource in resources[
        'items'] if resource['path'] == '/'][0]['id']

    client.put_method(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        authorizationType='none',
    )

    client.put_method_response(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        statusCode='200',
    )

    response = client.put_integration(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        type='HTTP',
        uri='http://httpbin.org/robots.txt',
    )

    response = client.put_integration_response(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        statusCode='200',
        selectionPattern='foobar',
    )
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response.should.equal({
        'statusCode': '200',
        'selectionPattern': 'foobar',
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'responseTemplates': {
            'application/json': None
        }
    })

    response = client.get_integration_response(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        statusCode='200',
    )
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response.should.equal({
        'statusCode': '200',
        'selectionPattern': 'foobar',
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'responseTemplates': {
            'application/json': None
        }
    })

    response = client.get_method(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
    )
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response['methodIntegration']['integrationResponses'].should.equal({
        '200': {
            'responseTemplates': {
                'application/json': None
            },
            'selectionPattern': 'foobar',
            'statusCode': '200'
        }
    })

    response = client.delete_integration_response(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        statusCode='200',
    )

    response = client.get_method(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
    )
    response['methodIntegration']['integrationResponses'].should.equal({})


@mock_apigateway
def test_update_stage_configuration():
    client = boto3.client('apigateway', region_name='us-west-2')
    stage_name = 'staging'
    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']

    response = client.create_deployment(
        restApiId=api_id,
        stageName=stage_name,
        description="1.0.1"
    )
    deployment_id = response['id']

    response = client.get_deployment(
        restApiId=api_id,
        deploymentId=deployment_id,
    )
    # createdDate is hard to match against, remove it
    response.pop('createdDate', None)
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response.should.equal({
        'id': deployment_id,
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'description': '1.0.1'
    })

    response = client.create_deployment(
        restApiId=api_id,
        stageName=stage_name,
        description="1.0.2"
    )
    deployment_id2 = response['id']

    stage = client.get_stage(
        restApiId=api_id,
        stageName=stage_name
    )
    stage['stageName'].should.equal(stage_name)
    stage['deploymentId'].should.equal(deployment_id2)
    stage.shouldnt.have.key('cacheClusterSize')

    client.update_stage(restApiId=api_id, stageName=stage_name,
                        patchOperations=[
                            {
                                "op": "replace",
                                "path": "/cacheClusterEnabled",
                                "value": "True"
                            }
                        ])

    stage = client.get_stage(
        restApiId=api_id,
        stageName=stage_name
    )

    stage.should.have.key('cacheClusterSize').which.should.equal("0.5")

    client.update_stage(restApiId=api_id, stageName=stage_name,
                        patchOperations=[
                            {
                                "op": "replace",
                                "path": "/cacheClusterSize",
                                "value": "1.6"
                            }
                        ])

    stage = client.get_stage(
        restApiId=api_id,
        stageName=stage_name
    )

    stage.should.have.key('cacheClusterSize').which.should.equal("1.6")

    client.update_stage(restApiId=api_id, stageName=stage_name,
                        patchOperations=[
                            {
                                "op": "replace",
                                "path": "/deploymentId",
                                "value": deployment_id
                            },
                            {
                                "op": "replace",
                                "path": "/variables/environment",
                                "value": "dev"
                            },
                            {
                                "op": "replace",
                                "path": "/variables/region",
                                "value": "eu-west-1"
                            },
                            {
                                "op": "replace",
                                "path": "/*/*/caching/dataEncrypted",
                                "value": "True"
                            },
                            {
                                "op": "replace",
                                "path": "/cacheClusterEnabled",
                                "value": "True"
                            },
                            {
                                "op": "replace",
                                "path": "/description",
                                "value": "stage description update"
                            },
                            {
                                "op": "replace",
                                "path": "/cacheClusterSize",
                                "value": "1.6"
                            }
                        ])

    client.update_stage(restApiId=api_id, stageName=stage_name,
                        patchOperations=[
                            {
                                "op": "remove",
                                "path": "/variables/region",
                                "value": "eu-west-1"
                            }
                        ])

    stage = client.get_stage(restApiId=api_id, stageName=stage_name)

    stage['description'].should.match('stage description update')
    stage['cacheClusterSize'].should.equal("1.6")
    stage['variables']['environment'].should.match('dev')
    stage['variables'].should_not.have.key('region')
    stage['cacheClusterEnabled'].should.be.true
    stage['deploymentId'].should.match(deployment_id)
    stage['methodSettings'].should.have.key('*/*')
    stage['methodSettings'][
        '*/*'].should.have.key('cacheDataEncrypted').which.should.be.true

    try:
        client.update_stage(restApiId=api_id, stageName=stage_name,
                            patchOperations=[
                                {
                                    "op": "add",
                                    "path": "/notasetting",
                                    "value": "eu-west-1"
                                }
                            ])
        assert False.should.be.ok  # Fail, should not be here
    except Exception:
        assert True.should.be.ok


@mock_apigateway
def test_non_existent_stage():
    client = boto3.client('apigateway', region_name='us-west-2')
    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']

    client.get_stage.when.called_with(
        restApiId=api_id, stageName='xxx').should.throw(ClientError)


@mock_apigateway
def test_create_stage():
    client = boto3.client('apigateway', region_name='us-west-2')
    stage_name = 'staging'
    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']

    response = client.create_deployment(
        restApiId=api_id,
        stageName=stage_name,
    )
    deployment_id = response['id']

    response = client.get_deployment(
        restApiId=api_id,
        deploymentId=deployment_id,
    )
    # createdDate is hard to match against, remove it
    response.pop('createdDate', None)
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response.should.equal({
        'id': deployment_id,
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'description': ''
    })

    response = client.create_deployment(
        restApiId=api_id,
        stageName=stage_name,
    )

    deployment_id2 = response['id']

    response = client.get_deployments(
        restApiId=api_id,
    )

    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)

    response['items'][0].pop('createdDate')
    response['items'][1].pop('createdDate')
    response['items'][0]['id'].should.match(
        r"{0}|{1}".format(deployment_id2, deployment_id))
    response['items'][1]['id'].should.match(
        r"{0}|{1}".format(deployment_id2, deployment_id))

    new_stage_name = 'current'
    response = client.create_stage(
        restApiId=api_id, stageName=new_stage_name, deploymentId=deployment_id2)

    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)

    response.should.equal({
        'stageName': new_stage_name,
        'deploymentId': deployment_id2,
        'methodSettings': {},
        'variables': {},
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'description': '',
        'cacheClusterEnabled': False
    })

    stage = client.get_stage(
        restApiId=api_id,
        stageName=new_stage_name
    )
    stage['stageName'].should.equal(new_stage_name)
    stage['deploymentId'].should.equal(deployment_id2)

    new_stage_name_with_vars = 'stage_with_vars'
    response = client.create_stage(restApiId=api_id, stageName=new_stage_name_with_vars, deploymentId=deployment_id2, variables={
        "env": "dev"
    })

    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)

    response.should.equal({
        'stageName': new_stage_name_with_vars,
        'deploymentId': deployment_id2,
        'methodSettings': {},
        'variables': {"env": "dev"},
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'description': '',
        'cacheClusterEnabled': False
    })

    stage = client.get_stage(
        restApiId=api_id,
        stageName=new_stage_name_with_vars
    )
    stage['stageName'].should.equal(new_stage_name_with_vars)
    stage['deploymentId'].should.equal(deployment_id2)
    stage['variables'].should.have.key('env').which.should.match("dev")

    new_stage_name = 'stage_with_vars_and_cache_settings'
    response = client.create_stage(restApiId=api_id, stageName=new_stage_name, deploymentId=deployment_id2, variables={
        "env": "dev"
    }, cacheClusterEnabled=True, description="hello moto")

    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)

    response.should.equal({
        'stageName': new_stage_name,
        'deploymentId': deployment_id2,
        'methodSettings': {},
        'variables': {"env": "dev"},
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'description': 'hello moto',
        'cacheClusterEnabled': True,
        'cacheClusterSize': "0.5"
    })

    stage = client.get_stage(
        restApiId=api_id,
        stageName=new_stage_name
    )

    stage['cacheClusterSize'].should.equal("0.5")

    new_stage_name = 'stage_with_vars_and_cache_settings_and_size'
    response = client.create_stage(restApiId=api_id, stageName=new_stage_name, deploymentId=deployment_id2, variables={
        "env": "dev"
    }, cacheClusterEnabled=True, cacheClusterSize="1.6", description="hello moto")

    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)

    response.should.equal({
        'stageName': new_stage_name,
        'deploymentId': deployment_id2,
        'methodSettings': {},
        'variables': {"env": "dev"},
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'description': 'hello moto',
        'cacheClusterEnabled': True,
        'cacheClusterSize': "1.6"
    })

    stage = client.get_stage(
        restApiId=api_id,
        stageName=new_stage_name
    )
    stage['stageName'].should.equal(new_stage_name)
    stage['deploymentId'].should.equal(deployment_id2)
    stage['variables'].should.have.key('env').which.should.match("dev")
    stage['cacheClusterSize'].should.equal("1.6")


@mock_apigateway
def test_deployment():
    client = boto3.client('apigateway', region_name='us-west-2')
    stage_name = 'staging'
    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']

    response = client.create_deployment(
        restApiId=api_id,
        stageName=stage_name,
    )
    deployment_id = response['id']

    response = client.get_deployment(
        restApiId=api_id,
        deploymentId=deployment_id,
    )
    # createdDate is hard to match against, remove it
    response.pop('createdDate', None)
    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response.should.equal({
        'id': deployment_id,
        'ResponseMetadata': {'HTTPStatusCode': 200},
        'description': ''
    })

    response = client.get_deployments(
        restApiId=api_id,
    )

    response['items'][0].pop('createdDate')
    response['items'].should.equal([
        {'id': deployment_id, 'description': ''}
    ])

    response = client.delete_deployment(
        restApiId=api_id,
        deploymentId=deployment_id,
    )

    response = client.get_deployments(
        restApiId=api_id,
    )
    len(response['items']).should.equal(0)

    # test deployment stages

    stage = client.get_stage(
        restApiId=api_id,
        stageName=stage_name
    )
    stage['stageName'].should.equal(stage_name)
    stage['deploymentId'].should.equal(deployment_id)

    stage = client.update_stage(
        restApiId=api_id,
        stageName=stage_name,
        patchOperations=[
            {
                'op': 'replace',
                'path': '/description',
                'value': '_new_description_'
            },
        ]
    )

    stage = client.get_stage(
        restApiId=api_id,
        stageName=stage_name
    )
    stage['stageName'].should.equal(stage_name)
    stage['deploymentId'].should.equal(deployment_id)
    stage['description'].should.equal('_new_description_')


@mock_apigateway
def test_http_proxying_integration():
    responses.add(
        responses.GET, "http://httpbin.org/robots.txt", body='a fake response'
    )

    region_name = 'us-west-2'
    client = boto3.client('apigateway', region_name=region_name)
    response = client.create_rest_api(
        name='my_api',
        description='this is my api',
    )
    api_id = response['id']

    resources = client.get_resources(restApiId=api_id)
    root_id = [resource for resource in resources[
        'items'] if resource['path'] == '/'][0]['id']

    client.put_method(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        authorizationType='none',
    )

    client.put_method_response(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        statusCode='200',
    )

    response = client.put_integration(
        restApiId=api_id,
        resourceId=root_id,
        httpMethod='GET',
        type='HTTP',
        uri='http://httpbin.org/robots.txt',
    )

    stage_name = 'staging'
    client.create_deployment(
        restApiId=api_id,
        stageName=stage_name,
    )

    deploy_url = "https://{api_id}.execute-api.{region_name}.amazonaws.com/{stage_name}".format(
        api_id=api_id, region_name=region_name, stage_name=stage_name)

    if not settings.TEST_SERVER_MODE:
        requests.get(deploy_url).content.should.equal(b"a fake response")



from __future__ import unicode_literals
import boto3
import sure  # noqa
from moto import mock_ec2


@mock_ec2
def test_describe_nat_gateways():
    conn = boto3.client('ec2', 'us-east-1')

    response = conn.describe_nat_gateways()

    response['NatGateways'].should.have.length_of(0)


@mock_ec2
def test_create_nat_gateway():
    conn = boto3.client('ec2', 'us-east-1')
    vpc = conn.create_vpc(CidrBlock='10.0.0.0/16')
    vpc_id = vpc['Vpc']['VpcId']
    subnet = conn.create_subnet(
        VpcId=vpc_id,
        CidrBlock='10.0.1.0/27',
        AvailabilityZone='us-east-1a',
    )
    allocation_id = conn.allocate_address(Domain='vpc')['AllocationId']
    subnet_id = subnet['Subnet']['SubnetId']

    response = conn.create_nat_gateway(
        SubnetId=subnet_id,
        AllocationId=allocation_id,
    )

    response['NatGateway']['VpcId'].should.equal(vpc_id)
    response['NatGateway']['SubnetId'].should.equal(subnet_id)
    response['NatGateway']['State'].should.equal('available')


@mock_ec2
def test_delete_nat_gateway():
    conn = boto3.client('ec2', 'us-east-1')
    vpc = conn.create_vpc(CidrBlock='10.0.0.0/16')
    vpc_id = vpc['Vpc']['VpcId']
    subnet = conn.create_subnet(
        VpcId=vpc_id,
        CidrBlock='10.0.1.0/27',
        AvailabilityZone='us-east-1a',
    )
    allocation_id = conn.allocate_address(Domain='vpc')['AllocationId']
    subnet_id = subnet['Subnet']['SubnetId']

    nat_gateway = conn.create_nat_gateway(
        SubnetId=subnet_id,
        AllocationId=allocation_id,
    )
    nat_gateway_id = nat_gateway['NatGateway']['NatGatewayId']
    response = conn.delete_nat_gateway(NatGatewayId=nat_gateway_id)

    # this is hard to match against, so remove it
    response['ResponseMetadata'].pop('HTTPHeaders', None)
    response['ResponseMetadata'].pop('RetryAttempts', None)
    response.should.equal({
        'NatGatewayId': nat_gateway_id,
        'ResponseMetadata': {
            'HTTPStatusCode': 200,
            'RequestId': '741fc8ab-6ebe-452b-b92b-example'
        }
    })


@mock_ec2
def test_create_and_describe_nat_gateway():
    conn = boto3.client('ec2', 'us-east-1')
    vpc = conn.create_vpc(CidrBlock='10.0.0.0/16')
    vpc_id = vpc['Vpc']['VpcId']
    subnet = conn.create_subnet(
        VpcId=vpc_id,
        CidrBlock='10.0.1.0/27',
        AvailabilityZone='us-east-1a',
    )
    allocation_id = conn.allocate_address(Domain='vpc')['AllocationId']
    subnet_id = subnet['Subnet']['SubnetId']

    create_response = conn.create_nat_gateway(
        SubnetId=subnet_id,
        AllocationId=allocation_id,
    )
    nat_gateway_id = create_response['NatGateway']['NatGatewayId']
    describe_response = conn.describe_nat_gateways()

    enis = conn.describe_network_interfaces()['NetworkInterfaces']
    eni_id = enis[0]['NetworkInterfaceId']
    public_ip = conn.describe_addresses(AllocationIds=[allocation_id])[
        'Addresses'][0]['PublicIp']

    describe_response['NatGateways'].should.have.length_of(1)
    describe_response['NatGateways'][0][
        'NatGatewayId'].should.equal(nat_gateway_id)
    describe_response['NatGateways'][0]['State'].should.equal('available')
    describe_response['NatGateways'][0]['SubnetId'].should.equal(subnet_id)
    describe_response['NatGateways'][0]['VpcId'].should.equal(vpc_id)
    describe_response['NatGateways'][0]['NatGatewayAddresses'][
        0]['AllocationId'].should.equal(allocation_id)
    describe_response['NatGateways'][0]['NatGatewayAddresses'][
        0]['NetworkInterfaceId'].should.equal(eni_id)
    assert describe_response['NatGateways'][0][
        'NatGatewayAddresses'][0]['PrivateIp'].startswith('10.')
    describe_response['NatGateways'][0]['NatGatewayAddresses'][
        0]['PublicIp'].should.equal(public_ip)



from __future__ import unicode_literals

import json

from six.moves.urllib.parse import parse_qs

import boto3
import re
from freezegun import freeze_time
import sure  # noqa

from moto.packages.responses import responses
from botocore.exceptions import ClientError
from moto import mock_sns, mock_sqs
from freezegun import freeze_time


MESSAGE_FROM_SQS_TEMPLATE = '{\n  "Message": "%s",\n  "MessageId": "%s",\n  "Signature": "EXAMPLElDMXvB8r9R83tGoNn0ecwd5UjllzsvSvbItzfaMpN2nk5HVSw7XnOn/49IkxDKz8YrlH2qJXj2iZB0Zo2O71c4qQk1fMUDi3LGpij7RCW7AW9vYYsSqIKRnFS94ilu7NFhUzLiieYr4BKHpdTmdD6c0esKEYBpabxDSc=",\n  "SignatureVersion": "1",\n  "SigningCertURL": "https://sns.us-east-1.amazonaws.com/SimpleNotificationService-f3ecfb7224c7233fe7bb5f59f96de52f.pem",\n  "Subject": "my subject",\n  "Timestamp": "2015-01-01T12:00:00.000Z",\n  "TopicArn": "arn:aws:sns:%s:123456789012:some-topic",\n  "Type": "Notification",\n  "UnsubscribeURL": "https://sns.us-east-1.amazonaws.com/?Action=Unsubscribe&SubscriptionArn=arn:aws:sns:us-east-1:123456789012:some-topic:2bcfbf39-05c3-41de-beaa-fcfcc21c8f55"\n}'


@mock_sqs
@mock_sns
def test_publish_to_sqs():
    conn = boto3.client('sns', region_name='us-east-1')
    conn.create_topic(Name="some-topic")
    response = conn.list_topics()
    topic_arn = response["Topics"][0]['TopicArn']

    sqs_conn = boto3.resource('sqs', region_name='us-east-1')
    sqs_conn.create_queue(QueueName="test-queue")

    conn.subscribe(TopicArn=topic_arn,
                   Protocol="sqs",
                   Endpoint="arn:aws:sqs:us-east-1:123456789012:test-queue")
    message = 'my message'
    with freeze_time("2015-01-01 12:00:00"):
        published_message = conn.publish(TopicArn=topic_arn, Message=message)
    published_message_id = published_message['MessageId']

    queue = sqs_conn.get_queue_by_name(QueueName="test-queue")
    messages = queue.receive_messages(MaxNumberOfMessages=1)
    expected = MESSAGE_FROM_SQS_TEMPLATE  % (message, published_message_id, 'us-east-1')
    acquired_message = re.sub("\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z", u'2015-01-01T12:00:00.000Z', messages[0].body)
    acquired_message.should.equal(expected)


@mock_sns
def test_publish_sms():
    client = boto3.client('sns', region_name='us-east-1')
    client.create_topic(Name="some-topic")
    resp = client.create_topic(Name="some-topic")
    arn = resp['TopicArn']

    client.subscribe(
        TopicArn=arn,
        Protocol='sms',
        Endpoint='+15551234567'
    )

    result = client.publish(PhoneNumber="+15551234567", Message="my message")
    result.should.contain('MessageId')


@mock_sns
def test_publish_bad_sms():
    client = boto3.client('sns', region_name='us-east-1')
    client.create_topic(Name="some-topic")
    resp = client.create_topic(Name="some-topic")
    arn = resp['TopicArn']

    client.subscribe(
        TopicArn=arn,
        Protocol='sms',
        Endpoint='+15551234567'
    )

    try:
        # Test invalid number
        client.publish(PhoneNumber="NAA+15551234567", Message="my message")
    except ClientError as err:
        err.response['Error']['Code'].should.equal('InvalidParameter')

    try:
        # Test not found number
        client.publish(PhoneNumber="+44001234567", Message="my message")
    except ClientError as err:
        err.response['Error']['Code'].should.equal('ParameterValueInvalid')


@mock_sqs
@mock_sns
def test_publish_to_sqs_dump_json():
    conn = boto3.client('sns', region_name='us-east-1')
    conn.create_topic(Name="some-topic")
    response = conn.list_topics()
    topic_arn = response["Topics"][0]['TopicArn']

    sqs_conn = boto3.resource('sqs', region_name='us-east-1')
    sqs_conn.create_queue(QueueName="test-queue")

    conn.subscribe(TopicArn=topic_arn,
                   Protocol="sqs",
                   Endpoint="arn:aws:sqs:us-east-1:123456789012:test-queue")

    message = json.dumps({
        "Records": [{
            "eventVersion": "2.0",
            "eventSource": "aws:s3",
            "s3": {
                "s3SchemaVersion": "1.0"
            }
        }]
    }, sort_keys=True)
    with freeze_time("2015-01-01 12:00:00"):
        published_message = conn.publish(TopicArn=topic_arn, Message=message)
    published_message_id = published_message['MessageId']

    queue = sqs_conn.get_queue_by_name(QueueName="test-queue")
    messages = queue.receive_messages(MaxNumberOfMessages=1)

    escaped = message.replace('"', '\\"')
    expected = MESSAGE_FROM_SQS_TEMPLATE  % (escaped, published_message_id, 'us-east-1')
    acquired_message = re.sub("\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z", u'2015-01-01T12:00:00.000Z', messages[0].body)
    acquired_message.should.equal(expected)


@mock_sqs
@mock_sns
def test_publish_to_sqs_in_different_region():
    conn = boto3.client('sns', region_name='us-west-1')
    conn.create_topic(Name="some-topic")
    response = conn.list_topics()
    topic_arn = response["Topics"][0]['TopicArn']

    sqs_conn = boto3.resource('sqs', region_name='us-west-2')
    sqs_conn.create_queue(QueueName="test-queue")

    conn.subscribe(TopicArn=topic_arn,
                   Protocol="sqs",
                   Endpoint="arn:aws:sqs:us-west-2:123456789012:test-queue")

    message = 'my message'
    with freeze_time("2015-01-01 12:00:00"):
        published_message = conn.publish(TopicArn=topic_arn, Message=message)
    published_message_id = published_message['MessageId']

    queue = sqs_conn.get_queue_by_name(QueueName="test-queue")
    messages = queue.receive_messages(MaxNumberOfMessages=1)
    expected = MESSAGE_FROM_SQS_TEMPLATE  % (message, published_message_id, 'us-west-1')
    acquired_message = re.sub("\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z", u'2015-01-01T12:00:00.000Z', messages[0].body)
    acquired_message.should.equal(expected)


@freeze_time("2013-01-01")
@mock_sns
def test_publish_to_http():
    def callback(request):
        request.headers["Content-Type"].should.equal("application/json")
        json.loads.when.called_with(request.body).should_not.throw(Exception)
        return 200, {}, ""

    responses.add_callback(
        method="POST",
        url="http://example.com/foobar",
        callback=callback,
    )

    conn = boto3.client('sns', region_name='us-east-1')
    conn.create_topic(Name="some-topic")
    response = conn.list_topics()
    topic_arn = response["Topics"][0]['TopicArn']

    conn.subscribe(TopicArn=topic_arn,
                   Protocol="http",
                   Endpoint="http://example.com/foobar")

    response = conn.publish(
        TopicArn=topic_arn, Message="my message", Subject="my subject")
    message_id = response['MessageId']



from __future__ import unicode_literals

import json
import re

try:
    from urllib import unquote
    from urlparse import urlparse, parse_qs
except:
    from urllib.parse import unquote, urlparse, parse_qs

from moto.core.responses import BaseResponse


class LambdaResponse(BaseResponse):

    def root(self, request, full_url, headers):
        self.setup_class(request, full_url, headers)
        if request.method == 'GET':
            return self._list_functions(request, full_url, headers)
        elif request.method == 'POST':
            return self._create_function(request, full_url, headers)
        else:
            raise ValueError("Cannot handle request")

    def function(self, request, full_url, headers):
        self.setup_class(request, full_url, headers)
        if request.method == 'GET':
            return self._get_function(request, full_url, headers)
        elif request.method == 'DELETE':
            return self._delete_function(request, full_url, headers)
        else:
            raise ValueError("Cannot handle request")

    def invoke(self, request, full_url, headers):
        self.setup_class(request, full_url, headers)
        if request.method == 'POST':
            return self._invoke(request, full_url)
        else:
            raise ValueError("Cannot handle request")

    def invoke_async(self, request, full_url, headers):
        self.setup_class(request, full_url, headers)
        if request.method == 'POST':
            return self._invoke_async(request, full_url)
        else:
            raise ValueError("Cannot handle request")

    def tag(self, request, full_url, headers):
        self.setup_class(request, full_url, headers)
        if request.method == 'GET':
            return self._list_tags(request, full_url)
        elif request.method == 'POST':
            return self._tag_resource(request, full_url)
        elif request.method == 'DELETE':
            return self._untag_resource(request, full_url)
        else:
            raise ValueError("Cannot handle {0} request".format(request.method))

    def _invoke(self, request, full_url):
        response_headers = {}
        lambda_backend = self.get_lambda_backend(full_url)

        path = request.path if hasattr(request, 'path') else request.path_url
        function_name = path.split('/')[-2]

        if lambda_backend.has_function(function_name):
            fn = lambda_backend.get_function(function_name)
            payload = fn.invoke(self.body, self.headers, response_headers)
            response_headers['Content-Length'] = str(len(payload))
            return 202, response_headers, payload
        else:
            return 404, response_headers, "{}"

    def _invoke_async(self, request, full_url):
        response_headers = {}
        lambda_backend = self.get_lambda_backend(full_url)

        path = request.path if hasattr(request, 'path') else request.path_url
        function_name = path.split('/')[-3]
        if lambda_backend.has_function(function_name):
            fn = lambda_backend.get_function(function_name)
            fn.invoke(self.body, self.headers, response_headers)
            response_headers['Content-Length'] = str(0)
            return 202, response_headers, ""
        else:
            return 404, response_headers, "{}"

    def _list_functions(self, request, full_url, headers):
        lambda_backend = self.get_lambda_backend(full_url)
        return 200, {}, json.dumps({
            "Functions": [fn.get_configuration() for fn in lambda_backend.list_functions()],
            # "NextMarker": str(uuid.uuid4()),
        })

    def _create_function(self, request, full_url, headers):
        lambda_backend = self.get_lambda_backend(full_url)
        spec = json.loads(self.body)
        try:
            fn = lambda_backend.create_function(spec)
        except ValueError as e:
            return 400, {}, json.dumps({"Error": {"Code": e.args[0], "Message": e.args[1]}})
        else:
            config = fn.get_configuration()
            return 201, {}, json.dumps(config)

    def _delete_function(self, request, full_url, headers):
        lambda_backend = self.get_lambda_backend(full_url)

        path = request.path if hasattr(request, 'path') else request.path_url
        function_name = path.split('/')[-1]

        if lambda_backend.has_function(function_name):
            lambda_backend.delete_function(function_name)
            return 204, {}, ""
        else:
            return 404, {}, "{}"

    def _get_function(self, request, full_url, headers):
        lambda_backend = self.get_lambda_backend(full_url)

        path = request.path if hasattr(request, 'path') else request.path_url
        function_name = path.split('/')[-1]

        if lambda_backend.has_function(function_name):
            fn = lambda_backend.get_function(function_name)
            code = fn.get_code()
            return 200, {}, json.dumps(code)
        else:
            return 404, {}, "{}"

    def get_lambda_backend(self, full_url):
        from moto.awslambda.models import lambda_backends
        region = self._get_aws_region(full_url)
        return lambda_backends[region]

    def _get_aws_region(self, full_url):
        region = re.search(self.region_regex, full_url)
        if region:
            return region.group(1)
        else:
            return self.default_region

    def _list_tags(self, request, full_url):
        lambda_backend = self.get_lambda_backend(full_url)

        path = request.path if hasattr(request, 'path') else request.path_url
        function_arn = unquote(path.split('/')[-1])

        if lambda_backend.has_function_arn(function_arn):
            function = lambda_backend.get_function_by_arn(function_arn)
            return 200, {}, json.dumps(dict(Tags=function.tags))
        else:
            return 404, {}, "{}"

    def _tag_resource(self, request, full_url):
        lambda_backend = self.get_lambda_backend(full_url)

        path = request.path if hasattr(request, 'path') else request.path_url
        function_arn = unquote(path.split('/')[-1])

        spec = json.loads(self.body)

        if lambda_backend.has_function_arn(function_arn):
            lambda_backend.tag_resource(function_arn, spec['Tags'])
            return 200, {}, "{}"
        else:
            return 404, {}, "{}"

    def _untag_resource(self, request, full_url):
        lambda_backend = self.get_lambda_backend(full_url)

        path = request.path if hasattr(request, 'path') else request.path_url
        function_arn = unquote(path.split('/')[-1].split('?')[0])

        tag_keys = parse_qs(urlparse(full_url).query)['tagKeys']

        if lambda_backend.has_function_arn(function_arn):
            lambda_backend.untag_resource(function_arn, tag_keys)
            return 204, {}, "{}"
        else:
            return 404, {}, "{}"



#!/usr/bin/env python
from __future__ import unicode_literals
import setuptools
from setuptools import setup, find_packages
import sys


install_requires = [
    "Jinja2>=2.8",
    "boto>=2.36.0",
    "boto3>=1.2.1",
    "cookies",
    "cryptography>=2.0.0",
    "requests>=2.5",
    "xmltodict",
    "six>1.9",
    "werkzeug",
    "pyaml",
    "pytz",
    "python-dateutil<3.0.0,>=2.1",
    "mock",
    "docker>=2.5.1"
]

extras_require = {
    'server': ['flask'],
}

# https://hynek.me/articles/conditional-python-dependencies/
if int(setuptools.__version__.split(".", 1)[0]) < 18:
    if sys.version_info[0:2] < (3, 3):
        install_requires.append("backports.tempfile")
else:
    extras_require[":python_version<'3.3'"] = ["backports.tempfile"]


setup(
    name='moto',
    version='1.1.21',
    description='A library that allows your python tests to easily'
                ' mock out the boto library',
    author='Steve Pulec',
    author_email='spulec@gmail.com',
    url='https://github.com/spulec/moto',
    entry_points={
        'console_scripts': [
            'moto_server = moto.server:main',
        ],
    },
    packages=find_packages(exclude=("tests", "tests.*")),
    install_requires=install_requires,
    extras_require=extras_require,
    include_package_data=True,
    license="Apache",
    test_suite="tests",
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: Apache Software License",
        "Topic :: Software Development :: Testing",
    ],
)



from __future__ import unicode_literals

import base64
import botocore.client
import boto3
import hashlib
import io
import json
import zipfile
import sure  # noqa

from freezegun import freeze_time
from moto import mock_lambda, mock_s3, mock_ec2, settings

_lambda_region = 'us-east-1' if settings.TEST_SERVER_MODE else 'us-west-2'


def _process_lambda(func_str):
    zip_output = io.BytesIO()
    zip_file = zipfile.ZipFile(zip_output, 'w', zipfile.ZIP_DEFLATED)
    zip_file.writestr('lambda_function.py', func_str)
    zip_file.close()
    zip_output.seek(0)
    return zip_output.read()


def get_test_zip_file1():
    pfunc = """
def lambda_handler(event, context):
    return event
"""
    return _process_lambda(pfunc)


def get_test_zip_file2():
    func_str = """
import boto3

def lambda_handler(event, context):
    ec2 = boto3.resource('ec2', region_name='us-west-2', endpoint_url='http://{base_url}')

    volume_id = event.get('volume_id')
    vol = ec2.Volume(volume_id)

    print('get volume details for %s\\nVolume - %s  state=%s, size=%s' % (volume_id, volume_id, vol.state, vol.size))
    return event
""".format(base_url="motoserver:5000" if settings.TEST_SERVER_MODE else "ec2.us-west-2.amazonaws.com")
    return _process_lambda(func_str)


@mock_lambda
def test_list_functions():
    conn = boto3.client('lambda', 'us-west-2')
    result = conn.list_functions()
    result['Functions'].should.have.length_of(0)


@mock_lambda
def test_invoke_requestresponse_function():
    conn = boto3.client('lambda', 'us-west-2')
    conn.create_function(
        FunctionName='testFunction',
        Runtime='python2.7',
        Role='test-iam-role',
        Handler='lambda_function.lambda_handler',
        Code={
            'ZipFile': get_test_zip_file1(),
        },
        Description='test lambda function',
        Timeout=3,
        MemorySize=128,
        Publish=True,
    )

    in_data = {'msg': 'So long and thanks for all the fish'}
    success_result = conn.invoke(FunctionName='testFunction', InvocationType='RequestResponse',
                                 Payload=json.dumps(in_data))

    success_result["StatusCode"].should.equal(202)
    result_obj = json.loads(
        base64.b64decode(success_result["LogResult"]).decode('utf-8'))

    result_obj.should.equal(in_data)

    payload = success_result["Payload"].read().decode('utf-8')
    json.loads(payload).should.equal(in_data)


@mock_lambda
def test_invoke_event_function():
    conn = boto3.client('lambda', 'us-west-2')
    conn.create_function(
        FunctionName='testFunction',
        Runtime='python2.7',
        Role='test-iam-role',
        Handler='lambda_function.lambda_handler',
        Code={
            'ZipFile': get_test_zip_file1(),
        },
        Description='test lambda function',
        Timeout=3,
        MemorySize=128,
        Publish=True,
    )

    conn.invoke.when.called_with(
        FunctionName='notAFunction',
        InvocationType='Event',
        Payload='{}'
    ).should.throw(botocore.client.ClientError)

    in_data = {'msg': 'So long and thanks for all the fish'}
    success_result = conn.invoke(
        FunctionName='testFunction', InvocationType='Event', Payload=json.dumps(in_data))
    success_result["StatusCode"].should.equal(202)
    json.loads(success_result['Payload'].read().decode(
        'utf-8')).should.equal({})


if settings.TEST_SERVER_MODE:
    @mock_ec2
    @mock_lambda
    def test_invoke_function_get_ec2_volume():
        conn = boto3.resource("ec2", "us-west-2")
        vol = conn.create_volume(Size=99, AvailabilityZone='us-west-2')
        vol = conn.Volume(vol.id)

        conn = boto3.client('lambda', 'us-west-2')
        conn.create_function(
            FunctionName='testFunction',
            Runtime='python2.7',
            Role='test-iam-role',
            Handler='lambda_function.lambda_handler',
            Code={
                'ZipFile': get_test_zip_file2(),
            },
            Description='test lambda function',
            Timeout=3,
            MemorySize=128,
            Publish=True,
        )

        in_data = {'volume_id': vol.id}
        result = conn.invoke(FunctionName='testFunction',
                             InvocationType='RequestResponse', Payload=json.dumps(in_data))
        result["StatusCode"].should.equal(202)
        msg = 'get volume details for %s\nVolume - %s  state=%s, size=%s\n%s' % (
            vol.id, vol.id, vol.state, vol.size, json.dumps(in_data))

        log_result = base64.b64decode(result["LogResult"]).decode('utf-8')

        # fix for running under travis (TODO: investigate why it has an extra newline)
        log_result = log_result.replace('\n\n', '\n')
        log_result.should.equal(msg)

        payload = result['Payload'].read().decode('utf-8')

        # fix for running under travis (TODO: investigate why it has an extra newline)
        payload = payload.replace('\n\n', '\n')
        payload.should.equal(msg)


@mock_lambda
def test_create_based_on_s3_with_missing_bucket():
    conn = boto3.client('lambda', 'us-west-2')

    conn.create_function.when.called_with(
        FunctionName='testFunction',
        Runtime='python2.7',
        Role='test-iam-role',
        Handler='lambda_function.lambda_handler',
        Code={
            'S3Bucket': 'this-bucket-does-not-exist',
            'S3Key': 'test.zip',
        },
        Description='test lambda function',
        Timeout=3,
        MemorySize=128,
        Publish=True,
        VpcConfig={
            "SecurityGroupIds": ["sg-123abc"],
            "SubnetIds": ["subnet-123abc"],
        },
    ).should.throw(botocore.client.ClientError)


@mock_lambda
@mock_s3
@freeze_time('2015-01-01 00:00:00')
def test_create_function_from_aws_bucket():
    s3_conn = boto3.client('s3', 'us-west-2')
    s3_conn.create_bucket(Bucket='test-bucket')
    zip_content = get_test_zip_file2()

    s3_conn.put_object(Bucket='test-bucket', Key='test.zip', Body=zip_content)
    conn = boto3.client('lambda', 'us-west-2')

    result = conn.create_function(
        FunctionName='testFunction',
        Runtime='python2.7',
        Role='test-iam-role',
        Handler='lambda_function.lambda_handler',
        Code={
            'S3Bucket': 'test-bucket',
            'S3Key': 'test.zip',
        },
        Description='test lambda function',
        Timeout=3,
        MemorySize=128,
        Publish=True,
        VpcConfig={
            "SecurityGroupIds": ["sg-123abc"],
            "SubnetIds": ["subnet-123abc"],
        },
    )
    # this is hard to match against, so remove it
    result['ResponseMetadata'].pop('HTTPHeaders', None)
    # Botocore inserts retry attempts not seen in Python27
    result['ResponseMetadata'].pop('RetryAttempts', None)
    result.pop('LastModified')
    result.should.equal({
        'FunctionName': 'testFunction',
        'FunctionArn': 'arn:aws:lambda:{}:123456789012:function:testFunction'.format(_lambda_region),
        'Runtime': 'python2.7',
        'Role': 'test-iam-role',
        'Handler': 'lambda_function.lambda_handler',
        "CodeSha256": hashlib.sha256(zip_content).hexdigest(),
        "CodeSize": len(zip_content),
        'Description': 'test lambda function',
        'Timeout': 3,
        'MemorySize': 128,
        'Version': '$LATEST',
        'VpcConfig': {
            "SecurityGroupIds": ["sg-123abc"],
            "SubnetIds": ["subnet-123abc"],
            "VpcId": "vpc-123abc"
        },
        'ResponseMetadata': {'HTTPStatusCode': 201},
    })


@mock_lambda
@freeze_time('2015-01-01 00:00:00')
def test_create_function_from_zipfile():
    conn = boto3.client('lambda', 'us-west-2')
    zip_content = get_test_zip_file1()
    result = conn.create_function(
        FunctionName='testFunction',
        Runtime='python2.7',
        Role='test-iam-role',
        Handler='lambda_function.lambda_handler',
        Code={
            'ZipFile': zip_content,
        },
        Description='test lambda function',
        Timeout=3,
        MemorySize=128,
        Publish=True,
    )
    # this is hard to match against, so remove it
    result['ResponseMetadata'].pop('HTTPHeaders', None)
    # Botocore inserts retry attempts not seen in Python27
    result['ResponseMetadata'].pop('RetryAttempts', None)
    result.pop('LastModified')

    result.should.equal({
        'FunctionName': 'testFunction',
        'FunctionArn': 'arn:aws:lambda:{}:123456789012:function:testFunction'.format(_lambda_region),
        'Runtime': 'python2.7',
        'Role': 'test-iam-role',
        'Handler': 'lambda_function.lambda_handler',
        'CodeSize': len(zip_content),
        'Description': 'test lambda function',
        'Timeout': 3,
        'MemorySize': 128,
        'CodeSha256': hashlib.sha256(zip_content).hexdigest(),
        'Version': '$LATEST',
        'VpcConfig': {
            "SecurityGroupIds": [],
            "SubnetIds": [],
        },

        'ResponseMetadata': {'HTTPStatusCode': 201},
    })


@mock_lambda
@mock_s3
@freeze_time('2015-01-01 00:00:00')
def test_get_function():
    s3_conn = boto3.client('s3', 'us-west-2')
    s3_conn.create_bucket(Bucket='test-bucket')

    zip_content = get_test_zip_file1()
    s3_conn.put_object(Bucket='test-bucket', Key='test.zip', Body=zip_content)
    conn = boto3.client('lambda', 'us-west-2')

    conn.create_function(
        FunctionName='testFunction',
        Runtime='python2.7',
        Role='test-iam-role',
        Handler='lambda_function.lambda_handler',
        Code={
            'S3Bucket': 'test-bucket',
            'S3Key': 'test.zip',
        },
        Description='test lambda function',
        Timeout=3,
        MemorySize=128,
        Publish=True,
    )

    result = conn.get_function(FunctionName='testFunction')
    # this is hard to match against, so remove it
    result['ResponseMetadata'].pop('HTTPHeaders', None)
    # Botocore inserts retry attempts not seen in Python27
    result['ResponseMetadata'].pop('RetryAttempts', None)
    result['Configuration'].pop('LastModified')

    result.should.equal({
        "Code": {
            "Location": "s3://awslambda-{0}-tasks.s3-{0}.amazonaws.com/test.zip".format(_lambda_region),
            "RepositoryType": "S3"
        },
        "Configuration": {
            "CodeSha256": hashlib.sha256(zip_content).hexdigest(),
            "CodeSize": len(zip_content),
            "Description": "test lambda function",
            "FunctionArn": 'arn:aws:lambda:{}:123456789012:function:testFunction'.format(_lambda_region),
            "FunctionName": "testFunction",
            "Handler": "lambda_function.lambda_handler",
            "MemorySize": 128,
            "Role": "test-iam-role",
            "Runtime": "python2.7",
            "Timeout": 3,
            "Version": '$LATEST',
            "VpcConfig": {
                "SecurityGroupIds": [],
                "SubnetIds": [],
            }
        },
        'ResponseMetadata': {'HTTPStatusCode': 200},
    })


@mock_lambda
@mock_s3
def test_delete_function():
    s3_conn = boto3.client('s3', 'us-west-2')
    s3_conn.create_bucket(Bucket='test-bucket')

    zip_content = get_test_zip_file2()
    s3_conn.put_object(Bucket='test-bucket', Key='test.zip', Body=zip_content)
    conn = boto3.client('lambda', 'us-west-2')

    conn.create_function(
        FunctionName='testFunction',
        Runtime='python2.7',
        Role='test-iam-role',
        Handler='lambda_function.lambda_handler',
        Code={
            'S3Bucket': 'test-bucket',
            'S3Key': 'test.zip',
        },
        Description='test lambda function',
        Timeout=3,
        MemorySize=128,
        Publish=True,
    )

    success_result = conn.delete_function(FunctionName='testFunction')
    # this is hard to match against, so remove it
    success_result['ResponseMetadata'].pop('HTTPHeaders', None)
    # Botocore inserts retry attempts not seen in Python27
    success_result['ResponseMetadata'].pop('RetryAttempts', None)

    success_result.should.equal({'ResponseMetadata': {'HTTPStatusCode': 204}})

    conn.delete_function.when.called_with(
        FunctionName='testFunctionThatDoesntExist').should.throw(botocore.client.ClientError)


@mock_lambda
@mock_s3
@freeze_time('2015-01-01 00:00:00')
def test_list_create_list_get_delete_list():
    """
    test `list -> create -> list -> get -> delete -> list` integration

    """
    s3_conn = boto3.client('s3', 'us-west-2')
    s3_conn.create_bucket(Bucket='test-bucket')

    zip_content = get_test_zip_file2()
    s3_conn.put_object(Bucket='test-bucket', Key='test.zip', Body=zip_content)
    conn = boto3.client('lambda', 'us-west-2')

    conn.list_functions()['Functions'].should.have.length_of(0)

    conn.create_function(
        FunctionName='testFunction',
        Runtime='python2.7',
        Role='test-iam-role',
        Handler='lambda_function.lambda_handler',
        Code={
            'S3Bucket': 'test-bucket',
            'S3Key': 'test.zip',
        },
        Description='test lambda function',
        Timeout=3,
        MemorySize=128,
        Publish=True,
    )
    expected_function_result = {
        "Code": {
            "Location": "s3://awslambda-{0}-tasks.s3-{0}.amazonaws.com/test.zip".format(_lambda_region),
            "RepositoryType": "S3"
        },
        "Configuration": {
            "CodeSha256": hashlib.sha256(zip_content).hexdigest(),
            "CodeSize": len(zip_content),
            "Description": "test lambda function",
            "FunctionArn": 'arn:aws:lambda:{}:123456789012:function:testFunction'.format(_lambda_region),
            "FunctionName": "testFunction",
            "Handler": "lambda_function.lambda_handler",
            "MemorySize": 128,
            "Role": "test-iam-role",
            "Runtime": "python2.7",
            "Timeout": 3,
            "Version": '$LATEST',
            "VpcConfig": {
                "SecurityGroupIds": [],
                "SubnetIds": [],
            }
        },
        'ResponseMetadata': {'HTTPStatusCode': 200},
    }
    func = conn.list_functions()['Functions'][0]
    func.pop('LastModified')
    func.should.equal(expected_function_result['Configuration'])

    func = conn.get_function(FunctionName='testFunction')
    # this is hard to match against, so remove it
    func['ResponseMetadata'].pop('HTTPHeaders', None)
    # Botocore inserts retry attempts not seen in Python27
    func['ResponseMetadata'].pop('RetryAttempts', None)
    func['Configuration'].pop('LastModified')

    func.should.equal(expected_function_result)
    conn.delete_function(FunctionName='testFunction')

    conn.list_functions()['Functions'].should.have.length_of(0)


@mock_lambda
def test_invoke_lambda_error():
    lambda_fx = """
def lambda_handler(event, context):
    raise Exception('failsauce')
    """
    zip_output = io.BytesIO()
    zip_file = zipfile.ZipFile(zip_output, 'w', zipfile.ZIP_DEFLATED)
    zip_file.writestr('lambda_function.py', lambda_fx)
    zip_file.close()
    zip_output.seek(0)

    client = boto3.client('lambda', region_name='us-east-1')
    client.create_function(
        FunctionName='test-lambda-fx',
        Runtime='python2.7',
        Role='test-iam-role',
        Handler='lambda_function.lambda_handler',
        Description='test lambda function',
        Timeout=3,
        MemorySize=128,
        Publish=True,
        Code={
            'ZipFile': zip_output.read()
        },
    )

    result = client.invoke(
        FunctionName='test-lambda-fx',
        InvocationType='RequestResponse',
        LogType='Tail'
    )

    assert 'FunctionError' in result
    assert result['FunctionError'] == 'Handled'

@mock_lambda
@mock_s3
def test_tags():
    """
    test list_tags -> tag_resource -> list_tags -> tag_resource -> list_tags -> untag_resource -> list_tags integration
    """
    s3_conn = boto3.client('s3', 'us-west-2')
    s3_conn.create_bucket(Bucket='test-bucket')

    zip_content = get_test_zip_file2()
    s3_conn.put_object(Bucket='test-bucket', Key='test.zip', Body=zip_content)
    conn = boto3.client('lambda', 'us-west-2')

    function = conn.create_function(
        FunctionName='testFunction',
        Runtime='python2.7',
        Role='test-iam-role',
        Handler='lambda_function.handler',
        Code={
            'S3Bucket': 'test-bucket',
            'S3Key': 'test.zip',
        },
        Description='test lambda function',
        Timeout=3,
        MemorySize=128,
        Publish=True,
    )

    # List tags when there are none
    conn.list_tags(
        Resource=function['FunctionArn']
    )['Tags'].should.equal(dict())

    # List tags when there is one
    conn.tag_resource(
        Resource=function['FunctionArn'],
        Tags=dict(spam='eggs')
    )['ResponseMetadata']['HTTPStatusCode'].should.equal(200)
    conn.list_tags(
        Resource=function['FunctionArn']
    )['Tags'].should.equal(dict(spam='eggs'))

    # List tags when another has been added
    conn.tag_resource(
        Resource=function['FunctionArn'],
        Tags=dict(foo='bar')
    )['ResponseMetadata']['HTTPStatusCode'].should.equal(200)
    conn.list_tags(
        Resource=function['FunctionArn']
    )['Tags'].should.equal(dict(spam='eggs', foo='bar'))

    # Untag resource
    conn.untag_resource(
        Resource=function['FunctionArn'],
        TagKeys=['spam', 'trolls']
    )['ResponseMetadata']['HTTPStatusCode'].should.equal(204)
    conn.list_tags(
        Resource=function['FunctionArn']
    )['Tags'].should.equal(dict(foo='bar'))

    # Untag a tag that does not exist (no error and no change)
    conn.untag_resource(
        Resource=function['FunctionArn'],
        TagKeys=['spam']
    )['ResponseMetadata']['HTTPStatusCode'].should.equal(204)

@mock_lambda
def test_tags_not_found():
    """
    Test list_tags and tag_resource when the lambda with the given arn does not exist
    """
    conn = boto3.client('lambda', 'us-west-2')
    conn.list_tags.when.called_with(
        Resource='arn:aws:lambda:123456789012:function:not-found'
    ).should.throw(botocore.client.ClientError)

    conn.tag_resource.when.called_with(
        Resource='arn:aws:lambda:123456789012:function:not-found',
        Tags=dict(spam='eggs')
    ).should.throw(botocore.client.ClientError)

    conn.untag_resource.when.called_with(
        Resource='arn:aws:lambda:123456789012:function:not-found',
        TagKeys=['spam']
    ).should.throw(botocore.client.ClientError)

@mock_lambda
def test_invoke_async_function():
    conn = boto3.client('lambda', 'us-west-2')
    conn.create_function(
        FunctionName='testFunction',
        Runtime='python2.7',
        Role='test-iam-role',
        Handler='lambda_function.handler',
        Code={
            'ZipFile': get_test_zip_file1(),
        },
        Description='test lambda function',
        Timeout=3,
        MemorySize=128,
        Publish=True,
    )

    success_result = conn.invoke_async(
        FunctionName='testFunction', 
        InvokeArgs=json.dumps({ 'test': 'event' })
        )

    success_result['Status'].should.equal(202)

@mock_lambda
@freeze_time('2015-01-01 00:00:00')
def test_get_function_created_with_zipfile():
    conn = boto3.client('lambda', 'us-west-2')
    zip_content = get_test_zip_file1()
    result = conn.create_function(
        FunctionName='testFunction',
        Runtime='python2.7',
        Role='test-iam-role',
        Handler='lambda_function.handler',
        Code={
            'ZipFile': zip_content,
        },
        Description='test lambda function',
        Timeout=3,
        MemorySize=128,
        Publish=True,
    )

    response = conn.get_function(
        FunctionName='testFunction'
    )
    response['Configuration'].pop('LastModified')

    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)
    assert len(response['Code']) == 2
    assert response['Code']['RepositoryType'] == 'S3'
    assert response['Code']['Location'].startswith('s3://awslambda-{0}-tasks.s3-{0}.amazonaws.com'.format(_lambda_region))
    response['Configuration'].should.equal(
        {
            "CodeSha256": hashlib.sha256(zip_content).hexdigest(),
            "CodeSize": len(zip_content),
            "Description": "test lambda function",
            "FunctionArn":'arn:aws:lambda:{}:123456789012:function:testFunction'.format(_lambda_region),
            "FunctionName": "testFunction",
            "Handler": "lambda_function.handler",
            "MemorySize": 128,
            "Role": "test-iam-role",
            "Runtime": "python2.7",
            "Timeout": 3,
            "Version": '$LATEST',
            "VpcConfig": {
                "SecurityGroupIds": [],
                "SubnetIds": [],
            }
        },
    )



from __future__ import unicode_literals

import boto
from boto.s3.connection import OrdinaryCallingFormat

from moto import mock_s3_deprecated


def create_connection(key=None, secret=None):
    return boto.connect_s3(key, secret, calling_format=OrdinaryCallingFormat())


def test_bucketpath_combo_serial():
    @mock_s3_deprecated
    def make_bucket_path():
        conn = create_connection()
        conn.create_bucket('mybucketpath')

    @mock_s3_deprecated
    def make_bucket():
        conn = boto.connect_s3('the_key', 'the_secret')
        conn.create_bucket('mybucket')

    make_bucket()
    make_bucket_path()



from __future__ import unicode_literals
import boto3
import json

import sure  # noqa

from botocore.exceptions import ClientError
from nose.tools import assert_raises

from moto import mock_sns
from moto.sns.models import DEFAULT_PAGE_SIZE


@mock_sns
def test_subscribe_sms():
    client = boto3.client('sns', region_name='us-east-1')
    client.create_topic(Name="some-topic")
    resp = client.create_topic(Name="some-topic")
    arn = resp['TopicArn']

    resp = client.subscribe(
        TopicArn=arn,
        Protocol='sms',
        Endpoint='+15551234567'
    )
    resp.should.contain('SubscriptionArn')


@mock_sns
def test_subscribe_bad_sms():
    client = boto3.client('sns', region_name='us-east-1')
    client.create_topic(Name="some-topic")
    resp = client.create_topic(Name="some-topic")
    arn = resp['TopicArn']

    try:
        # Test invalid number
        client.subscribe(
            TopicArn=arn,
            Protocol='sms',
            Endpoint='NAA+15551234567'
        )
    except ClientError as err:
        err.response['Error']['Code'].should.equal('InvalidParameter')


@mock_sns
def test_creating_subscription():
    conn = boto3.client('sns', region_name='us-east-1')
    conn.create_topic(Name="some-topic")
    response = conn.list_topics()
    topic_arn = response["Topics"][0]['TopicArn']

    conn.subscribe(TopicArn=topic_arn,
                   Protocol="http",
                   Endpoint="http://example.com/")

    subscriptions = conn.list_subscriptions()["Subscriptions"]
    subscriptions.should.have.length_of(1)
    subscription = subscriptions[0]
    subscription["TopicArn"].should.equal(topic_arn)
    subscription["Protocol"].should.equal("http")
    subscription["SubscriptionArn"].should.contain(topic_arn)
    subscription["Endpoint"].should.equal("http://example.com/")

    # Now unsubscribe the subscription
    conn.unsubscribe(SubscriptionArn=subscription["SubscriptionArn"])

    # And there should be zero subscriptions left
    subscriptions = conn.list_subscriptions()["Subscriptions"]
    subscriptions.should.have.length_of(0)


@mock_sns
def test_deleting_subscriptions_by_deleting_topic():
    conn = boto3.client('sns', region_name='us-east-1')
    conn.create_topic(Name="some-topic")
    response = conn.list_topics()
    topic_arn = response["Topics"][0]['TopicArn']

    conn.subscribe(TopicArn=topic_arn,
                   Protocol="http",
                   Endpoint="http://example.com/")

    subscriptions = conn.list_subscriptions()["Subscriptions"]
    subscriptions.should.have.length_of(1)
    subscription = subscriptions[0]
    subscription["TopicArn"].should.equal(topic_arn)
    subscription["Protocol"].should.equal("http")
    subscription["SubscriptionArn"].should.contain(topic_arn)
    subscription["Endpoint"].should.equal("http://example.com/")

    # Now delete the topic
    conn.delete_topic(TopicArn=topic_arn)

    # And there should now be 0 topics
    topics_json = conn.list_topics()
    topics = topics_json["Topics"]
    topics.should.have.length_of(0)

    # And there should be zero subscriptions left
    subscriptions = conn.list_subscriptions()["Subscriptions"]
    subscriptions.should.have.length_of(0)


@mock_sns
def test_getting_subscriptions_by_topic():
    conn = boto3.client('sns', region_name='us-east-1')
    conn.create_topic(Name="topic1")
    conn.create_topic(Name="topic2")

    response = conn.list_topics()
    topics = response["Topics"]
    topic1_arn = topics[0]['TopicArn']
    topic2_arn = topics[1]['TopicArn']

    conn.subscribe(TopicArn=topic1_arn,
                   Protocol="http",
                   Endpoint="http://example1.com/")
    conn.subscribe(TopicArn=topic2_arn,
                   Protocol="http",
                   Endpoint="http://example2.com/")

    topic1_subscriptions = conn.list_subscriptions_by_topic(TopicArn=topic1_arn)[
        "Subscriptions"]
    topic1_subscriptions.should.have.length_of(1)
    topic1_subscriptions[0]['Endpoint'].should.equal("http://example1.com/")


@mock_sns
def test_subscription_paging():
    conn = boto3.client('sns', region_name='us-east-1')
    conn.create_topic(Name="topic1")

    response = conn.list_topics()
    topics = response["Topics"]
    topic1_arn = topics[0]['TopicArn']

    for index in range(DEFAULT_PAGE_SIZE + int(DEFAULT_PAGE_SIZE / 3)):
        conn.subscribe(TopicArn=topic1_arn,
                       Protocol='email',
                       Endpoint='email_' + str(index) + '@test.com')

    all_subscriptions = conn.list_subscriptions()
    all_subscriptions["Subscriptions"].should.have.length_of(DEFAULT_PAGE_SIZE)
    next_token = all_subscriptions["NextToken"]
    next_token.should.equal(str(DEFAULT_PAGE_SIZE))

    all_subscriptions = conn.list_subscriptions(NextToken=next_token)
    all_subscriptions["Subscriptions"].should.have.length_of(
        int(DEFAULT_PAGE_SIZE / 3))
    all_subscriptions.shouldnt.have("NextToken")

    topic1_subscriptions = conn.list_subscriptions_by_topic(
        TopicArn=topic1_arn)
    topic1_subscriptions["Subscriptions"].should.have.length_of(
        DEFAULT_PAGE_SIZE)
    next_token = topic1_subscriptions["NextToken"]
    next_token.should.equal(str(DEFAULT_PAGE_SIZE))

    topic1_subscriptions = conn.list_subscriptions_by_topic(
        TopicArn=topic1_arn, NextToken=next_token)
    topic1_subscriptions["Subscriptions"].should.have.length_of(
        int(DEFAULT_PAGE_SIZE / 3))
    topic1_subscriptions.shouldnt.have("NextToken")


@mock_sns
def test_set_subscription_attributes():
    conn = boto3.client('sns', region_name='us-east-1')
    conn.create_topic(Name="some-topic")
    response = conn.list_topics()
    topic_arn = response["Topics"][0]['TopicArn']

    conn.subscribe(TopicArn=topic_arn,
                   Protocol="http",
                   Endpoint="http://example.com/")

    subscriptions = conn.list_subscriptions()["Subscriptions"]
    subscriptions.should.have.length_of(1)
    subscription = subscriptions[0]
    subscription["TopicArn"].should.equal(topic_arn)
    subscription["Protocol"].should.equal("http")
    subscription["SubscriptionArn"].should.contain(topic_arn)
    subscription["Endpoint"].should.equal("http://example.com/")

    subscription_arn = subscription["SubscriptionArn"]
    attrs = conn.get_subscription_attributes(
        SubscriptionArn=subscription_arn
    )
    attrs.should.have.key('Attributes')
    conn.set_subscription_attributes(
        SubscriptionArn=subscription_arn,
        AttributeName='RawMessageDelivery',
        AttributeValue='true'
    )
    delivery_policy = json.dumps({
        'healthyRetryPolicy': {
            "numRetries": 10,
            "minDelayTarget": 1,
            "maxDelayTarget":2
        }
    })
    conn.set_subscription_attributes(
        SubscriptionArn=subscription_arn,
        AttributeName='DeliveryPolicy',
        AttributeValue=delivery_policy
    )
    attrs = conn.get_subscription_attributes(
        SubscriptionArn=subscription_arn
    )
    attrs['Attributes']['RawMessageDelivery'].should.equal('true')
    attrs['Attributes']['DeliveryPolicy'].should.equal(delivery_policy)

    # not existing subscription
    with assert_raises(ClientError):
        conn.set_subscription_attributes(
            SubscriptionArn='invalid',
            AttributeName='RawMessageDelivery',
            AttributeValue='true'
        )
    with assert_raises(ClientError):
        attrs = conn.get_subscription_attributes(
            SubscriptionArn='invalid'
        )


    # invalid attr name
    with assert_raises(ClientError):
        conn.set_subscription_attributes(
            SubscriptionArn=subscription_arn,
            AttributeName='InvalidName',
            AttributeValue='true'
        )


@mock_sns
def test_check_not_opted_out():
    conn = boto3.client('sns', region_name='us-east-1')
    response = conn.check_if_phone_number_is_opted_out(phoneNumber='+447428545375')

    response.should.contain('isOptedOut')
    response['isOptedOut'].should.be(False)


@mock_sns
def test_check_opted_out():
    # Phone number ends in 99 so is hardcoded in the endpoint to return opted
    # out status
    conn = boto3.client('sns', region_name='us-east-1')
    response = conn.check_if_phone_number_is_opted_out(phoneNumber='+447428545399')

    response.should.contain('isOptedOut')
    response['isOptedOut'].should.be(True)


@mock_sns
def test_check_opted_out_invalid():
    conn = boto3.client('sns', region_name='us-east-1')

    # Invalid phone number
    with assert_raises(ClientError):
        conn.check_if_phone_number_is_opted_out(phoneNumber='+44742LALALA')


@mock_sns
def test_list_opted_out():
    conn = boto3.client('sns', region_name='us-east-1')
    response = conn.list_phone_numbers_opted_out()

    response.should.contain('phoneNumbers')
    len(response['phoneNumbers']).should.be.greater_than(0)


@mock_sns
def test_opt_in():
    conn = boto3.client('sns', region_name='us-east-1')
    response = conn.list_phone_numbers_opted_out()
    current_len = len(response['phoneNumbers'])
    assert current_len > 0

    conn.opt_in_phone_number(phoneNumber=response['phoneNumbers'][0])

    response = conn.list_phone_numbers_opted_out()
    len(response['phoneNumbers']).should.be.greater_than(0)
    len(response['phoneNumbers']).should.be.lower_than(current_len)


@mock_sns
def test_confirm_subscription():
    conn = boto3.client('sns', region_name='us-east-1')
    response = conn.create_topic(Name='testconfirm')

    conn.confirm_subscription(
        TopicArn=response['TopicArn'],
        Token='2336412f37fb687f5d51e6e241d59b68c4e583a5cee0be6f95bbf97ab8d2441cf47b99e848408adaadf4c197e65f03473d53c4ba398f6abbf38ce2e8ebf7b4ceceb2cd817959bcde1357e58a2861b05288c535822eb88cac3db04f592285249971efc6484194fc4a4586147f16916692',
        AuthenticateOnUnsubscribe='true'
    )



from __future__ import unicode_literals

from botocore.exceptions import ClientError
import boto3
import sure  # noqa
from nose.tools import assert_raises
from moto import mock_polly

# Polly only available in a few regions
DEFAULT_REGION = 'eu-west-1'

LEXICON_XML = """<?xml version="1.0" encoding="UTF-8"?>
<lexicon version="1.0" 
      xmlns="http://www.w3.org/2005/01/pronunciation-lexicon"
      xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
      xsi:schemaLocation="http://www.w3.org/2005/01/pronunciation-lexicon 
        http://www.w3.org/TR/2007/CR-pronunciation-lexicon-20071212/pls.xsd"
      alphabet="ipa" 
      xml:lang="en-US">
  <lexeme>
    <grapheme>W3C</grapheme>
    <alias>World Wide Web Consortium</alias>
  </lexeme>
</lexicon>"""


@mock_polly
def test_describe_voices():
    client = boto3.client('polly', region_name=DEFAULT_REGION)

    resp = client.describe_voices()
    len(resp['Voices']).should.be.greater_than(1)

    resp = client.describe_voices(LanguageCode='en-GB')
    len(resp['Voices']).should.equal(3)

    try:
        client.describe_voices(LanguageCode='SOME_LANGUAGE')
    except ClientError as err:
        err.response['Error']['Code'].should.equal('400')
    else:
        raise RuntimeError('Should of raised an exception')


@mock_polly
def test_put_list_lexicon():
    client = boto3.client('polly', region_name=DEFAULT_REGION)

    # Return nothing
    client.put_lexicon(
        Name='test',
        Content=LEXICON_XML
    )

    resp = client.list_lexicons()
    len(resp['Lexicons']).should.equal(1)


@mock_polly
def test_put_get_lexicon():
    client = boto3.client('polly', region_name=DEFAULT_REGION)

    # Return nothing
    client.put_lexicon(
        Name='test',
        Content=LEXICON_XML
    )

    resp = client.get_lexicon(Name='test')
    resp.should.contain('Lexicon')
    resp.should.contain('LexiconAttributes')


@mock_polly
def test_put_lexicon_bad_name():
    client = boto3.client('polly', region_name=DEFAULT_REGION)

    try:
        client.put_lexicon(
            Name='test-invalid',
            Content=LEXICON_XML
        )
    except ClientError as err:
        err.response['Error']['Code'].should.equal('InvalidParameterValue')
    else:
        raise RuntimeError('Should of raised an exception')


@mock_polly
def test_synthesize_speech():
    client = boto3.client('polly', region_name=DEFAULT_REGION)

    # Return nothing
    client.put_lexicon(
        Name='test',
        Content=LEXICON_XML
    )

    tests = (
        ('pcm', 'audio/pcm'),
        ('mp3', 'audio/mpeg'),
        ('ogg_vorbis', 'audio/ogg'),
    )
    for output_format, content_type in tests:
        resp = client.synthesize_speech(
            LexiconNames=['test'],
            OutputFormat=output_format,
            SampleRate='16000',
            Text='test1234',
            TextType='text',
            VoiceId='Astrid'
        )
        resp['ContentType'].should.equal(content_type)


@mock_polly
def test_synthesize_speech_bad_lexicon():
    client = boto3.client('polly', region_name=DEFAULT_REGION)
    client.put_lexicon(Name='test', Content=LEXICON_XML)

    try:
        client.synthesize_speech(
            LexiconNames=['test2'],
            OutputFormat='pcm',
            SampleRate='16000',
            Text='test1234',
            TextType='text',
            VoiceId='Astrid'
        )
    except ClientError as err:
        err.response['Error']['Code'].should.equal('LexiconNotFoundException')
    else:
        raise RuntimeError('Should of raised LexiconNotFoundException')


@mock_polly
def test_synthesize_speech_bad_output_format():
    client = boto3.client('polly', region_name=DEFAULT_REGION)
    client.put_lexicon(Name='test', Content=LEXICON_XML)

    try:
        client.synthesize_speech(
            LexiconNames=['test'],
            OutputFormat='invalid',
            SampleRate='16000',
            Text='test1234',
            TextType='text',
            VoiceId='Astrid'
        )
    except ClientError as err:
        err.response['Error']['Code'].should.equal('InvalidParameterValue')
    else:
        raise RuntimeError('Should of raised ')


@mock_polly
def test_synthesize_speech_bad_sample_rate():
    client = boto3.client('polly', region_name=DEFAULT_REGION)
    client.put_lexicon(Name='test', Content=LEXICON_XML)

    try:
        client.synthesize_speech(
            LexiconNames=['test'],
            OutputFormat='pcm',
            SampleRate='18000',
            Text='test1234',
            TextType='text',
            VoiceId='Astrid'
        )
    except ClientError as err:
        err.response['Error']['Code'].should.equal('InvalidSampleRateException')
    else:
        raise RuntimeError('Should of raised ')


@mock_polly
def test_synthesize_speech_bad_text_type():
    client = boto3.client('polly', region_name=DEFAULT_REGION)
    client.put_lexicon(Name='test', Content=LEXICON_XML)

    try:
        client.synthesize_speech(
            LexiconNames=['test'],
            OutputFormat='pcm',
            SampleRate='16000',
            Text='test1234',
            TextType='invalid',
            VoiceId='Astrid'
        )
    except ClientError as err:
        err.response['Error']['Code'].should.equal('InvalidParameterValue')
    else:
        raise RuntimeError('Should of raised ')


@mock_polly
def test_synthesize_speech_bad_voice_id():
    client = boto3.client('polly', region_name=DEFAULT_REGION)
    client.put_lexicon(Name='test', Content=LEXICON_XML)

    try:
        client.synthesize_speech(
            LexiconNames=['test'],
            OutputFormat='pcm',
            SampleRate='16000',
            Text='test1234',
            TextType='text',
            VoiceId='Luke'
        )
    except ClientError as err:
        err.response['Error']['Code'].should.equal('InvalidParameterValue')
    else:
        raise RuntimeError('Should of raised ')


@mock_polly
def test_synthesize_speech_text_too_long():
    client = boto3.client('polly', region_name=DEFAULT_REGION)
    client.put_lexicon(Name='test', Content=LEXICON_XML)

    try:
        client.synthesize_speech(
            LexiconNames=['test'],
            OutputFormat='pcm',
            SampleRate='16000',
            Text='test1234'*376,  # = 3008 characters
            TextType='text',
            VoiceId='Astrid'
        )
    except ClientError as err:
        err.response['Error']['Code'].should.equal('TextLengthExceededException')
    else:
        raise RuntimeError('Should of raised ')


@mock_polly
def test_synthesize_speech_bad_speech_marks1():
    client = boto3.client('polly', region_name=DEFAULT_REGION)
    client.put_lexicon(Name='test', Content=LEXICON_XML)

    try:
        client.synthesize_speech(
            LexiconNames=['test'],
            OutputFormat='pcm',
            SampleRate='16000',
            Text='test1234',
            TextType='text',
            SpeechMarkTypes=['word'],
            VoiceId='Astrid'
        )
    except ClientError as err:
        err.response['Error']['Code'].should.equal('MarksNotSupportedForFormatException')
    else:
        raise RuntimeError('Should of raised ')


@mock_polly
def test_synthesize_speech_bad_speech_marks2():
    client = boto3.client('polly', region_name=DEFAULT_REGION)
    client.put_lexicon(Name='test', Content=LEXICON_XML)

    try:
        client.synthesize_speech(
            LexiconNames=['test'],
            OutputFormat='pcm',
            SampleRate='16000',
            Text='test1234',
            TextType='ssml',
            SpeechMarkTypes=['word'],
            VoiceId='Astrid'
        )
    except ClientError as err:
        err.response['Error']['Code'].should.equal('MarksNotSupportedForFormatException')
    else:
        raise RuntimeError('Should of raised ')



from __future__ import unicode_literals
import sure  # noqa

from flask.testing import FlaskClient
import moto.server as server

'''
Test the different server responses
'''


class AuthenticatedClient(FlaskClient):
    def open(self, *args, **kwargs):
        kwargs['headers'] = kwargs.get('headers', {})
        kwargs['headers']['Authorization'] = "Any authorization header"
        return super(AuthenticatedClient, self).open(*args, **kwargs)


def authenticated_client():
    backend = server.create_backend_app("s3bucket_path")
    backend.test_client_class = AuthenticatedClient
    return backend.test_client()


def test_s3_server_get():
    test_client = authenticated_client()

    res = test_client.get('/')

    res.data.should.contain(b'ListAllMyBucketsResult')


def test_s3_server_bucket_create():
    test_client = authenticated_client()

    res = test_client.put('/foobar', 'http://localhost:5000')
    res.status_code.should.equal(200)

    res = test_client.get('/')
    res.data.should.contain(b'<Name>foobar</Name>')

    res = test_client.get('/foobar', 'http://localhost:5000')
    res.status_code.should.equal(200)
    res.data.should.contain(b"ListBucketResult")

    res = test_client.put('/foobar2/', 'http://localhost:5000')
    res.status_code.should.equal(200)

    res = test_client.get('/')
    res.data.should.contain(b'<Name>foobar2</Name>')

    res = test_client.get('/foobar2/', 'http://localhost:5000')
    res.status_code.should.equal(200)
    res.data.should.contain(b"ListBucketResult")

    res = test_client.get('/missing-bucket', 'http://localhost:5000')
    res.status_code.should.equal(404)

    res = test_client.put(
        '/foobar/bar', 'http://localhost:5000', data='test value')
    res.status_code.should.equal(200)

    res = test_client.get('/foobar/bar', 'http://localhost:5000')
    res.status_code.should.equal(200)
    res.data.should.equal(b"test value")


def test_s3_server_post_to_bucket():
    test_client = authenticated_client()

    res = test_client.put('/foobar2', 'http://localhost:5000/')
    res.status_code.should.equal(200)

    test_client.post('/foobar2', "https://localhost:5000/", data={
        'key': 'the-key',
        'file': 'nothing'
    })

    res = test_client.get('/foobar2/the-key', 'http://localhost:5000/')
    res.status_code.should.equal(200)
    res.data.should.equal(b"nothing")


def test_s3_server_put_ipv6():
    test_client = authenticated_client()

    res = test_client.put('/foobar2', 'http://[::]:5000/')
    res.status_code.should.equal(200)

    test_client.post('/foobar2', "https://[::]:5000/", data={
        'key': 'the-key',
        'file': 'nothing'
    })

    res = test_client.get('/foobar2/the-key', 'http://[::]:5000/')
    res.status_code.should.equal(200)
    res.data.should.equal(b"nothing")


def test_s3_server_put_ipv4():
    test_client = authenticated_client()

    res = test_client.put('/foobar2', 'http://127.0.0.1:5000/')
    res.status_code.should.equal(200)

    test_client.post('/foobar2', "https://127.0.0.1:5000/", data={
        'key': 'the-key',
        'file': 'nothing'
    })

    res = test_client.get('/foobar2/the-key', 'http://127.0.0.1:5000/')
    res.status_code.should.equal(200)
    res.data.should.equal(b"nothing")



from __future__ import unicode_literals

import boto3
from botocore.exceptions import ClientError
from moto import mock_sns
import sure  # noqa


@mock_sns
def test_create_platform_application():
    conn = boto3.client('sns', region_name='us-east-1')
    response = conn.create_platform_application(
        Name="my-application",
        Platform="APNS",
        Attributes={
            "PlatformCredential": "platform_credential",
            "PlatformPrincipal": "platform_principal",
        },
    )
    application_arn = response['PlatformApplicationArn']
    application_arn.should.equal(
        'arn:aws:sns:us-east-1:123456789012:app/APNS/my-application')


@mock_sns
def test_get_platform_application_attributes():
    conn = boto3.client('sns', region_name='us-east-1')
    platform_application = conn.create_platform_application(
        Name="my-application",
        Platform="APNS",
        Attributes={
            "PlatformCredential": "platform_credential",
            "PlatformPrincipal": "platform_principal",
        },
    )
    arn = platform_application['PlatformApplicationArn']
    attributes = conn.get_platform_application_attributes(
        PlatformApplicationArn=arn)['Attributes']
    attributes.should.equal({
        "PlatformCredential": "platform_credential",
        "PlatformPrincipal": "platform_principal",
    })


@mock_sns
def test_get_missing_platform_application_attributes():
    conn = boto3.client('sns', region_name='us-east-1')
    conn.get_platform_application_attributes.when.called_with(
        PlatformApplicationArn="a-fake-arn").should.throw(ClientError)


@mock_sns
def test_set_platform_application_attributes():
    conn = boto3.client('sns', region_name='us-east-1')
    platform_application = conn.create_platform_application(
        Name="my-application",
        Platform="APNS",
        Attributes={
            "PlatformCredential": "platform_credential",
            "PlatformPrincipal": "platform_principal",
        },
    )
    arn = platform_application['PlatformApplicationArn']
    conn.set_platform_application_attributes(PlatformApplicationArn=arn,
                                             Attributes={
                                                 "PlatformPrincipal": "other"}
                                             )
    attributes = conn.get_platform_application_attributes(
        PlatformApplicationArn=arn)['Attributes']
    attributes.should.equal({
        "PlatformCredential": "platform_credential",
        "PlatformPrincipal": "other",
    })


@mock_sns
def test_list_platform_applications():
    conn = boto3.client('sns', region_name='us-east-1')
    conn.create_platform_application(
        Name="application1",
        Platform="APNS",
        Attributes={},
    )
    conn.create_platform_application(
        Name="application2",
        Platform="APNS",
        Attributes={},
    )

    applications_repsonse = conn.list_platform_applications()
    applications = applications_repsonse['PlatformApplications']
    applications.should.have.length_of(2)


@mock_sns
def test_delete_platform_application():
    conn = boto3.client('sns', region_name='us-east-1')
    conn.create_platform_application(
        Name="application1",
        Platform="APNS",
        Attributes={},
    )
    conn.create_platform_application(
        Name="application2",
        Platform="APNS",
        Attributes={},
    )

    applications_repsonse = conn.list_platform_applications()
    applications = applications_repsonse['PlatformApplications']
    applications.should.have.length_of(2)

    application_arn = applications[0]['PlatformApplicationArn']
    conn.delete_platform_application(PlatformApplicationArn=application_arn)

    applications_repsonse = conn.list_platform_applications()
    applications = applications_repsonse['PlatformApplications']
    applications.should.have.length_of(1)


@mock_sns
def test_create_platform_endpoint():
    conn = boto3.client('sns', region_name='us-east-1')
    platform_application = conn.create_platform_application(
        Name="my-application",
        Platform="APNS",
        Attributes={},
    )
    application_arn = platform_application['PlatformApplicationArn']

    endpoint = conn.create_platform_endpoint(
        PlatformApplicationArn=application_arn,
        Token="some_unique_id",
        CustomUserData="some user data",
        Attributes={
            "Enabled": 'false',
        },
    )

    endpoint_arn = endpoint['EndpointArn']
    endpoint_arn.should.contain(
        "arn:aws:sns:us-east-1:123456789012:endpoint/APNS/my-application/")


@mock_sns
def test_create_duplicate_platform_endpoint():
    conn = boto3.client('sns', region_name='us-east-1')
    platform_application = conn.create_platform_application(
        Name="my-application",
        Platform="APNS",
        Attributes={},
    )
    application_arn = platform_application['PlatformApplicationArn']

    endpoint = conn.create_platform_endpoint(
        PlatformApplicationArn=application_arn,
        Token="some_unique_id",
        CustomUserData="some user data",
        Attributes={
            "Enabled": 'false',
        },
    )

    endpoint = conn.create_platform_endpoint.when.called_with(
        PlatformApplicationArn=application_arn,
        Token="some_unique_id",
        CustomUserData="some user data",
        Attributes={
            "Enabled": 'false',
        },
    ).should.throw(ClientError)


@mock_sns
def test_get_list_endpoints_by_platform_application():
    conn = boto3.client('sns', region_name='us-east-1')
    platform_application = conn.create_platform_application(
        Name="my-application",
        Platform="APNS",
        Attributes={},
    )
    application_arn = platform_application['PlatformApplicationArn']

    endpoint = conn.create_platform_endpoint(
        PlatformApplicationArn=application_arn,
        Token="some_unique_id",
        CustomUserData="some user data",
        Attributes={
            "CustomUserData": "some data",
        },
    )
    endpoint_arn = endpoint['EndpointArn']

    endpoint_list = conn.list_endpoints_by_platform_application(
        PlatformApplicationArn=application_arn
    )['Endpoints']

    endpoint_list.should.have.length_of(1)
    endpoint_list[0]['Attributes']['CustomUserData'].should.equal('some data')
    endpoint_list[0]['EndpointArn'].should.equal(endpoint_arn)


@mock_sns
def test_get_endpoint_attributes():
    conn = boto3.client('sns', region_name='us-east-1')
    platform_application = conn.create_platform_application(
        Name="my-application",
        Platform="APNS",
        Attributes={},
    )
    application_arn = platform_application['PlatformApplicationArn']

    endpoint = conn.create_platform_endpoint(
        PlatformApplicationArn=application_arn,
        Token="some_unique_id",
        CustomUserData="some user data",
        Attributes={
            "Enabled": 'false',
            "CustomUserData": "some data",
        },
    )
    endpoint_arn = endpoint['EndpointArn']

    attributes = conn.get_endpoint_attributes(
        EndpointArn=endpoint_arn)['Attributes']
    attributes.should.equal({
        "Token": "some_unique_id",
        "Enabled": 'false',
        "CustomUserData": "some data",
    })


@mock_sns
def test_get_missing_endpoint_attributes():
    conn = boto3.client('sns', region_name='us-east-1')
    conn.get_endpoint_attributes.when.called_with(
        EndpointArn="a-fake-arn").should.throw(ClientError)


@mock_sns
def test_set_endpoint_attributes():
    conn = boto3.client('sns', region_name='us-east-1')
    platform_application = conn.create_platform_application(
        Name="my-application",
        Platform="APNS",
        Attributes={},
    )
    application_arn = platform_application['PlatformApplicationArn']

    endpoint = conn.create_platform_endpoint(
        PlatformApplicationArn=application_arn,
        Token="some_unique_id",
        CustomUserData="some user data",
        Attributes={
            "Enabled": 'false',
            "CustomUserData": "some data",
        },
    )
    endpoint_arn = endpoint['EndpointArn']

    conn.set_endpoint_attributes(EndpointArn=endpoint_arn,
                                 Attributes={"CustomUserData": "other data"}
                                 )
    attributes = conn.get_endpoint_attributes(
        EndpointArn=endpoint_arn)['Attributes']
    attributes.should.equal({
        "Token": "some_unique_id",
        "Enabled": 'false',
        "CustomUserData": "other data",
    })


@mock_sns
def test_publish_to_platform_endpoint():
    conn = boto3.client('sns', region_name='us-east-1')
    platform_application = conn.create_platform_application(
        Name="my-application",
        Platform="APNS",
        Attributes={},
    )
    application_arn = platform_application['PlatformApplicationArn']

    endpoint = conn.create_platform_endpoint(
        PlatformApplicationArn=application_arn,
        Token="some_unique_id",
        CustomUserData="some user data",
        Attributes={
            "Enabled": 'true',
        },
    )

    endpoint_arn = endpoint['EndpointArn']

    conn.publish(Message="some message",
                 MessageStructure="json", TargetArn=endpoint_arn)


@mock_sns
def test_publish_to_disabled_platform_endpoint():
    conn = boto3.client('sns', region_name='us-east-1')
    platform_application = conn.create_platform_application(
        Name="my-application",
        Platform="APNS",
        Attributes={},
    )
    application_arn = platform_application['PlatformApplicationArn']

    endpoint = conn.create_platform_endpoint(
        PlatformApplicationArn=application_arn,
        Token="some_unique_id",
        CustomUserData="some user data",
        Attributes={
            "Enabled": 'false',
        },
    )

    endpoint_arn = endpoint['EndpointArn']

    conn.publish.when.called_with(
        Message="some message",
        MessageStructure="json",
        TargetArn=endpoint_arn,
    ).should.throw(ClientError)


@mock_sns
def test_set_sms_attributes():
    conn = boto3.client('sns', region_name='us-east-1')

    conn.set_sms_attributes(attributes={'DefaultSMSType': 'Transactional', 'test': 'test'})

    response = conn.get_sms_attributes()
    response.should.contain('attributes')
    response['attributes'].should.contain('DefaultSMSType')
    response['attributes'].should.contain('test')
    response['attributes']['DefaultSMSType'].should.equal('Transactional')
    response['attributes']['test'].should.equal('test')


@mock_sns
def test_get_sms_attributes_filtered():
    conn = boto3.client('sns', region_name='us-east-1')

    conn.set_sms_attributes(attributes={'DefaultSMSType': 'Transactional', 'test': 'test'})

    response = conn.get_sms_attributes(attributes=['DefaultSMSType'])
    response.should.contain('attributes')
    response['attributes'].should.contain('DefaultSMSType')
    response['attributes'].should_not.contain('test')
    response['attributes']['DefaultSMSType'].should.equal('Transactional')



from __future__ import unicode_literals
from moto.core.exceptions import RESTError


class RepositoryNotFoundException(RESTError):
    code = 400

    def __init__(self, repository_name, registry_id):
        super(RepositoryNotFoundException, self).__init__(
            error_type="RepositoryNotFoundException",
            message="The repository with name '{0}' does not exist in the registry "
                    "with id '{1}'".format(repository_name, registry_id))


class ImageNotFoundException(RESTError):
    code = 400

    def __init__(self, image_id, repository_name, registry_id):
        super(ImageNotFoundException, self).__init__(
            error_type="ImageNotFoundException",
            message="The image with imageId {0} does not exist within the repository with name '{1}' "
                    "in the registry with id '{2}'".format(image_id, repository_name, registry_id))



from __future__ import unicode_literals
import boto
import boto3
import sure  # noqa

from nose.tools import assert_raises
from boto.exception import BotoServerError
from moto import mock_iam, mock_iam_deprecated


@mock_iam_deprecated()
def test_create_group():
    conn = boto.connect_iam()
    conn.create_group('my-group')
    with assert_raises(BotoServerError):
        conn.create_group('my-group')


@mock_iam_deprecated()
def test_get_group():
    conn = boto.connect_iam()
    conn.create_group('my-group')
    conn.get_group('my-group')
    with assert_raises(BotoServerError):
        conn.get_group('not-group')


@mock_iam_deprecated()
def test_get_all_groups():
    conn = boto.connect_iam()
    conn.create_group('my-group1')
    conn.create_group('my-group2')
    groups = conn.get_all_groups()['list_groups_response'][
        'list_groups_result']['groups']
    groups.should.have.length_of(2)


@mock_iam_deprecated()
def test_add_user_to_group():
    conn = boto.connect_iam()
    with assert_raises(BotoServerError):
        conn.add_user_to_group('my-group', 'my-user')
    conn.create_group('my-group')
    with assert_raises(BotoServerError):
        conn.add_user_to_group('my-group', 'my-user')
    conn.create_user('my-user')
    conn.add_user_to_group('my-group', 'my-user')


@mock_iam_deprecated()
def test_remove_user_from_group():
    conn = boto.connect_iam()
    with assert_raises(BotoServerError):
        conn.remove_user_from_group('my-group', 'my-user')
    conn.create_group('my-group')
    conn.create_user('my-user')
    with assert_raises(BotoServerError):
        conn.remove_user_from_group('my-group', 'my-user')
    conn.add_user_to_group('my-group', 'my-user')
    conn.remove_user_from_group('my-group', 'my-user')


@mock_iam_deprecated()
def test_get_groups_for_user():
    conn = boto.connect_iam()
    conn.create_group('my-group1')
    conn.create_group('my-group2')
    conn.create_group('other-group')
    conn.create_user('my-user')
    conn.add_user_to_group('my-group1', 'my-user')
    conn.add_user_to_group('my-group2', 'my-user')

    groups = conn.get_groups_for_user(
        'my-user')['list_groups_for_user_response']['list_groups_for_user_result']['groups']
    groups.should.have.length_of(2)


@mock_iam_deprecated()
def test_put_group_policy():
    conn = boto.connect_iam()
    conn.create_group('my-group')
    conn.put_group_policy('my-group', 'my-policy', '{"some": "json"}')


@mock_iam
def test_attach_group_policies():
    conn = boto3.client('iam', region_name='us-east-1')
    conn.create_group(GroupName='my-group')
    conn.list_attached_group_policies(GroupName='my-group')['AttachedPolicies'].should.be.empty
    policy_arn = 'arn:aws:iam::aws:policy/service-role/AmazonElasticMapReduceforEC2Role'
    conn.list_attached_group_policies(GroupName='my-group')['AttachedPolicies'].should.be.empty
    conn.attach_group_policy(GroupName='my-group', PolicyArn=policy_arn)
    conn.list_attached_group_policies(GroupName='my-group')['AttachedPolicies'].should.equal(
        [
            {
                'PolicyName': 'AmazonElasticMapReduceforEC2Role',
                'PolicyArn': policy_arn,
            }
        ])

    conn.detach_group_policy(GroupName='my-group', PolicyArn=policy_arn)
    conn.list_attached_group_policies(GroupName='my-group')['AttachedPolicies'].should.be.empty


@mock_iam_deprecated()
def test_get_group_policy():
    conn = boto.connect_iam()
    conn.create_group('my-group')
    with assert_raises(BotoServerError):
        conn.get_group_policy('my-group', 'my-policy')

    conn.put_group_policy('my-group', 'my-policy', '{"some": "json"}')
    conn.get_group_policy('my-group', 'my-policy')


@mock_iam_deprecated()
def test_get_all_group_policies():
    conn = boto.connect_iam()
    conn.create_group('my-group')
    policies = conn.get_all_group_policies('my-group')['list_group_policies_response']['list_group_policies_result']['policy_names']
    assert policies == []
    conn.put_group_policy('my-group', 'my-policy', '{"some": "json"}')
    policies = conn.get_all_group_policies('my-group')['list_group_policies_response']['list_group_policies_result']['policy_names']
    assert policies == ['my-policy']


@mock_iam()
def test_list_group_policies():
    conn = boto3.client('iam', region_name='us-east-1')
    conn.create_group(GroupName='my-group')
    conn.list_group_policies(GroupName='my-group')['PolicyNames'].should.be.empty
    conn.put_group_policy(GroupName='my-group', PolicyName='my-policy', PolicyDocument='{"some": "json"}')
    conn.list_group_policies(GroupName='my-group')['PolicyNames'].should.equal(['my-policy'])



from __future__ import unicode_literals
from .responses import RDS2Response

url_bases = [
    "https?://rds.(.+).amazonaws.com",
    "https?://rds.amazonaws.com",
]

url_paths = {
    '{0}/$': RDS2Response.dispatch,
}



from __future__ import unicode_literals
import sure  # noqa
from nose.tools import assert_raises
import requests

import boto3
from moto import mock_sqs, settings

base_url = "http://localhost:5000" if settings.TEST_SERVER_MODE else "http://motoapi.amazonaws.com"


@mock_sqs
def test_reset_api():
    conn = boto3.client("sqs", region_name='us-west-1')
    conn.create_queue(QueueName="queue1")
    conn.list_queues()['QueueUrls'].should.have.length_of(1)

    res = requests.post("{base_url}/moto-api/reset".format(base_url=base_url))
    res.content.should.equal(b'{"status": "ok"}')

    conn.list_queues().shouldnt.contain('QueueUrls')  # No more queues


@mock_sqs
def test_data_api():
    conn = boto3.client("sqs", region_name='us-west-1')
    conn.create_queue(QueueName="queue1")

    res = requests.post("{base_url}/moto-api/data.json".format(base_url=base_url))
    queues = res.json()['sqs']['Queue']
    len(queues).should.equal(1)
    queue = queues[0]
    queue['name'].should.equal("queue1")



from __future__ import unicode_literals

import boto3
from botocore.exceptions import ClientError
import sure   # noqa

from moto import mock_cloudwatch


@mock_cloudwatch
def test_put_list_dashboard():
    client = boto3.client('cloudwatch', region_name='eu-central-1')
    widget = '{"widgets": [{"type": "text", "x": 0, "y": 7, "width": 3, "height": 3, "properties": {"markdown": "Hello world"}}]}'

    client.put_dashboard(DashboardName='test1', DashboardBody=widget)
    resp = client.list_dashboards()

    len(resp['DashboardEntries']).should.equal(1)


@mock_cloudwatch
def test_put_list_prefix_nomatch_dashboard():
    client = boto3.client('cloudwatch', region_name='eu-central-1')
    widget = '{"widgets": [{"type": "text", "x": 0, "y": 7, "width": 3, "height": 3, "properties": {"markdown": "Hello world"}}]}'

    client.put_dashboard(DashboardName='test1', DashboardBody=widget)
    resp = client.list_dashboards(DashboardNamePrefix='nomatch')

    len(resp['DashboardEntries']).should.equal(0)


@mock_cloudwatch
def test_delete_dashboard():
    client = boto3.client('cloudwatch', region_name='eu-central-1')
    widget = '{"widgets": [{"type": "text", "x": 0, "y": 7, "width": 3, "height": 3, "properties": {"markdown": "Hello world"}}]}'

    client.put_dashboard(DashboardName='test1', DashboardBody=widget)
    client.put_dashboard(DashboardName='test2', DashboardBody=widget)
    client.put_dashboard(DashboardName='test3', DashboardBody=widget)
    client.delete_dashboards(DashboardNames=['test2', 'test1'])

    resp = client.list_dashboards(DashboardNamePrefix='test3')
    len(resp['DashboardEntries']).should.equal(1)


@mock_cloudwatch
def test_delete_dashboard_fail():
    client = boto3.client('cloudwatch', region_name='eu-central-1')
    widget = '{"widgets": [{"type": "text", "x": 0, "y": 7, "width": 3, "height": 3, "properties": {"markdown": "Hello world"}}]}'

    client.put_dashboard(DashboardName='test1', DashboardBody=widget)
    client.put_dashboard(DashboardName='test2', DashboardBody=widget)
    client.put_dashboard(DashboardName='test3', DashboardBody=widget)
    # Doesnt delete anything if all dashboards to be deleted do not exist
    try:
        client.delete_dashboards(DashboardNames=['test2', 'test1', 'test_no_match'])
    except ClientError as err:
        err.response['Error']['Code'].should.equal('ResourceNotFound')
    else:
        raise RuntimeError('Should of raised error')

    resp = client.list_dashboards()
    len(resp['DashboardEntries']).should.equal(3)


@mock_cloudwatch
def test_get_dashboard():
    client = boto3.client('cloudwatch', region_name='eu-central-1')
    widget = '{"widgets": [{"type": "text", "x": 0, "y": 7, "width": 3, "height": 3, "properties": {"markdown": "Hello world"}}]}'
    client.put_dashboard(DashboardName='test1', DashboardBody=widget)

    resp = client.get_dashboard(DashboardName='test1')
    resp.should.contain('DashboardArn')
    resp.should.contain('DashboardBody')
    resp['DashboardName'].should.equal('test1')


@mock_cloudwatch
def test_get_dashboard_fail():
    client = boto3.client('cloudwatch', region_name='eu-central-1')

    try:
        client.get_dashboard(DashboardName='test1')
    except ClientError as err:
        err.response['Error']['Code'].should.equal('ResourceNotFound')
    else:
        raise RuntimeError('Should of raised error')







